package cn.com.qmth.stmms.api.controller;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.util.LinkedList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import cn.com.qmth.stmms.admin.utils.SessionExamUtils;
import cn.com.qmth.stmms.api.exception.ApiException;
import cn.com.qmth.stmms.biz.exam.bean.ResultMessage;
import cn.com.qmth.stmms.biz.exam.model.ExamSubject;
import cn.com.qmth.stmms.biz.exam.service.ExamSubjectService;
import cn.com.qmth.stmms.biz.school.model.School;
import cn.com.qmth.stmms.biz.school.service.SchoolService;
import cn.com.qmth.stmms.common.controller.BaseController;
import cn.com.qmth.stmms.common.domain.ApiUser;
import cn.com.qmth.stmms.common.utils.RequestUtils;
import net.sf.json.JSONObject;

public class BaseApiController extends BaseController {

    protected static final Logger log = LoggerFactory.getLogger(BaseApiController.class);

    // private static final String ERROR_MESSAGE_HEADER_KEY = "error-info";

    @Autowired
    private SchoolService schoolService;

    @Autowired
    private ExamSubjectService subjectService;

    // @ResponseBody
    // @ExceptionHandler(Exception.class)
    // public ResponseEntity<StatusResponse> handleException(Exception e,
    // HttpServletRequest request) {
    // StatusResponse body = new StatusResponse(500, "系统异常");
    // return asResult(e, body, request);
    // }
    //
    // @ResponseBody
    // @ExceptionHandler(RuntimeException.class)
    // public ResponseEntity<StatusResponse> handleException(RuntimeException e,
    // HttpServletRequest request,
    // HttpServletResponse response) {
    // StatusResponse body = null;
    //
    // if (e instanceof StatusException) {
    // StatusException se = (StatusException) e;
    // body = new StatusResponse(se.getCode(), se.getMessage());
    // } else if (e instanceof ApiException) {
    // response.addHeader(ERROR_MESSAGE_HEADER_KEY,
    // StringUtils.trimToEmpty(e.getMessage()));
    // ApiException teme = (ApiException) e;
    // try {
    // response.sendError(teme.getCode());
    // body = new StatusResponse(teme.getCode(), teme.getMessage());
    // } catch (IOException e1) {
    // log.error("api response senderror", e);
    // }
    // } else {
    // body = new StatusResponse(500, "系统异常");
    // }
    //
    // return asResult(e, body, request);
    // }
    //
    // private ResponseEntity<StatusResponse> asResult(Throwable err,
    // StatusResponse body, HttpServletRequest request) {
    //
    // HttpStatus httpStatus = HttpStatus.INTERNAL_SERVER_ERROR;
    //
    // log.error(err.getMessage(), err);
    //
    // HttpHeaders headers = new HttpHeaders();
    // headers.add("Content-Type", "application/json;charset=utf-8");
    // return new ResponseEntity<>(body, headers, httpStatus);
    // }

    protected void exportFile(String fileName, File file, HttpServletResponse response) {
        OutputStream out = null;
        InputStream in = null;
        try {
            in = new FileInputStream(file);
            fileName = URLEncoder.encode(fileName, "UTF-8");
            response.reset();
            response.setHeader("Content-Disposition", "inline; filename=" + fileName);
            response.addHeader("Content-Length", "" + file.length());
            response.setContentType("application/octet-stream;charset=UTF-8");
            out = new BufferedOutputStream(response.getOutputStream());
            IOUtils.copy(in, out);
            out.flush();
        } catch (IOException e) {
            throw new RuntimeException(e);
        } finally {
            IOUtils.closeQuietly(out);
            IOUtils.closeQuietly(in);
        }
    }

    protected ResultMessage resultOk() {
        ResultMessage ret = new ResultMessage();
        ret.setSuccess(true);
        return ret;
    }

    protected JSONObject result(boolean success) {
        JSONObject obj = new JSONObject();
        obj.accumulate("success", success);
        return obj;
    }

    protected JSONObject result(boolean success, String url) {
        JSONObject obj = new JSONObject();
        obj.accumulate("success", success);
        obj.accumulate("url", url);
        return obj;
    }

    protected JSONObject result(String message) {
        JSONObject obj = new JSONObject();
        obj.accumulate("success", true);
        obj.accumulate("message", message);
        return obj;
    }

    protected JSONObject result(String key, String value) {
        JSONObject obj = new JSONObject();
        obj.accumulate(key, value);
        return obj;
    }

    protected String validate(String name, String value, boolean required, int maxLength) {
        value = StringUtils.trimToNull(value);
        if (required && value == null) {
            throw ApiException.QUERY_PARAM_ERROR.replaceMessage(name + " is required");
        }
        if (value != null && maxLength > 0 && value.length() > maxLength) {
            throw ApiException.QUERY_PARAM_ERROR.replaceMessage(name + " length exceed " + maxLength);
        }
        return value;
    }

    protected String validate(String value, boolean notBlank, int maxLength) {
        if (value == null) {
            return null;
        }
        value = value.trim();
        if (notBlank && value.length() == 0) {
            return null;
        }
        if (maxLength > 0 && value.length() > maxLength) {
            return null;
        }
        return value;
    }

    protected School validate(Integer parentId, String subCode) {
        School school = schoolService.findById(parentId);
        if (StringUtils.isNotBlank(subCode)) {
            subCode = school.getCode() + "-" + subCode;
            school = schoolService.findByParentIdAndSubCode(parentId, subCode);
            if (school == null || !school.isEnable()) {
                throw ApiException.SCHOOL_NOT_ACCESSIBLED;
            }
        }
        return school;
    }

    protected boolean validateSchool(Integer parentId, Integer subId) {
        School sub = schoolService.findById(subId);
        if (sub == null) {
            return false;
        }
        return parentId.equals(subId) || sub.getParentId().equals(parentId);
    }

    protected Integer getSessionExamId(HttpServletRequest request) {
        return SessionExamUtils.getExamId(request);
    }

    protected List<ExamSubject> getExamSubject(Integer examId, ApiUser wu) {
        List<ExamSubject> list = new LinkedList<ExamSubject>();
        if (wu.isSchoolAdmin()) {
            list = subjectService.list(examId);
        } else if (wu.isSubjectHeader() || wu.isInspector() || wu.isSchoolViewer()) {
            list.addAll(subjectService.list(examId, wu.getSubjectCodeSet()));
        }
        return list;
    }

    protected boolean subjectCheck(String subjectCode, ApiUser wu) {
        if (wu.isSubjectHeader() || wu.isInspector() || wu.isSchoolViewer()) {
            for (String code : wu.getSubjectCodeSet()) {
                if (code.equalsIgnoreCase(subjectCode)) {
                    return true;
                }
            }
            return false;
        } else {
            return true;
        }
    }

    public ApiUser getApiUser() {
        return RequestUtils.getApiUser();
    }

    protected Integer getSessionExamId() {
        return SessionExamUtils.getExamId(RequestUtils.getRequest());
    }
}
