package cn.com.qmth.stmms.api.controller.admin;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

import com.qmth.boot.core.collection.PageResult;

import cn.com.qmth.stmms.api.controller.BaseApiController;
import cn.com.qmth.stmms.biz.exam.bean.ArbitrateHistoryVo;
import cn.com.qmth.stmms.biz.exam.model.ExamSubject;
import cn.com.qmth.stmms.biz.exam.model.MarkGroup;
import cn.com.qmth.stmms.biz.exam.service.ExamQuestionService;
import cn.com.qmth.stmms.biz.exam.service.MarkGroupService;
import com.qmth.boot.core.exception.StatusException;
import cn.com.qmth.stmms.biz.mark.model.ArbitrateHistory;
import cn.com.qmth.stmms.biz.mark.query.ArbitrateHistorySearchQuery;
import cn.com.qmth.stmms.biz.mark.service.ArbitrateHistoryService;
import cn.com.qmth.stmms.biz.user.service.UserService;
import cn.com.qmth.stmms.biz.utils.PageUtil;
import cn.com.qmth.stmms.common.domain.ApiUser;
import cn.com.qmth.stmms.common.enums.HistoryStatus;
import cn.com.qmth.stmms.common.enums.Role;
import cn.com.qmth.stmms.common.utils.ExportExcel;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@Api(tags = "仲裁管理")
@Controller("adminArbitrateController")
@RequestMapping("/api/admin/exam/arbitrate")
public class ArbitrateController extends BaseApiController {

    protected static Logger log = LoggerFactory.getLogger(ArbitrateController.class);

    public static final int UN_SELECTIVE_SCORE = -1;

    @Autowired
    private UserService userService;

    @Autowired
    private MarkGroupService groupService;

    @Autowired
    private ArbitrateHistoryService arbitrateService;

    @Autowired
    private ExamQuestionService questionService;

    @Value("${slice.split.config}")
    private String splitConfig;

    @Value("${marker.forceMode}")
    private String forceMarkMode;

    @ApiOperation(value = "状态")
    @RequestMapping(value = "status", method = RequestMethod.POST)
    @ResponseBody
    public List<HistoryStatus> statusLlist() {
        return HistoryStatus.getOptionList();
    }

    @ApiOperation(value = "分页查询")
    @RequestMapping(value = "list", method = RequestMethod.POST)
    @ResponseBody
    public PageResult<ArbitrateHistoryVo> list(ArbitrateHistorySearchQuery query) {
        return page(query);
    }

    private PageResult<ArbitrateHistoryVo> page(ArbitrateHistorySearchQuery query) {
        int examId = getSessionExamId();
        ApiUser wu = getApiUser();
        List<ExamSubject> subjectList = getExamSubject(examId, wu);
        if (subjectList.isEmpty()) {
            return PageUtil.emptyPage();
        }
        query.setExamId(examId);
        if (StringUtils.isBlank(query.getSubjectCode()) && !subjectList.isEmpty()) {
            query.setSubjectCode(subjectList.get(0).getCode());
        }
        List<MarkGroup> groupList = groupService.findByExamAndSubjectWithDouble(query.getExamId(),
                query.getSubjectCode());
        List<ArbitrateHistoryVo> ret = new ArrayList<>();
        if (!groupList.isEmpty()) {
            for (MarkGroup group : groupList) {
                group.setQuestionList(questionService.findByExamAndSubjectAndObjectiveAndGroupNumber(query.getExamId(),
                        group.getSubjectCode(), false, group.getNumber()));
            }
            query.addSort("examNumber", Direction.ASC);
            query = arbitrateService.findByQuery(query);
            for (ArbitrateHistory history : query.getResult()) {
                if (history.getUserId() != null) {
                    history.setUser(userService.findById(history.getUserId()));
                }
                ArbitrateHistoryVo vo = ArbitrateHistoryVo.of(history);
                if (Role.SUBJECT_HEADER.equals(wu.getRole()) && StringUtils.isNotBlank(history.getExamNumber())) {
                    vo.setExamNumber("***");
                }
                ret.add(vo);
            }

        }
        return PageUtil.of(ret, query);
    }

    @ApiOperation(value = "导出")
    @RequestMapping(value = "export", method = RequestMethod.POST)
    public void exportFile(ArbitrateHistorySearchQuery query, HttpServletResponse response) {
        String fileName = "仲裁数据.xlsx";
        query.setPageNumber(1);
        query.setPageSize(Integer.MAX_VALUE);
        PageResult<ArbitrateHistoryVo> page = page(query);
        try {
            new ExportExcel("仲裁数据", ArbitrateHistoryVo.class).setDataList(page.getResult()).write(response, fileName)
                    .dispose();
        } catch (IOException e) {
            throw new StatusException("仲裁数据导出出错", e);
        }
    }
}
