package cn.com.qmth.stmms.api.controller.admin;

import java.math.BigDecimal;
import java.util.*;

import cn.com.qmth.stmms.biz.exam.query.ExamStudentSearchQuery;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import com.qmth.boot.core.collection.PageResult;
import com.qmth.boot.core.exception.StatusException;

import cn.com.qmth.stmms.api.controller.BaseApiController;
import cn.com.qmth.stmms.biz.exam.bean.*;
import cn.com.qmth.stmms.biz.exam.model.Exam;
import cn.com.qmth.stmms.biz.exam.model.ExamStudent;
import cn.com.qmth.stmms.biz.exam.query.ExamSearchQuery;
import cn.com.qmth.stmms.biz.exam.service.CheckStudentService;
import cn.com.qmth.stmms.biz.exam.service.ExamService;
import cn.com.qmth.stmms.biz.exam.service.ExamStudentService;
import cn.com.qmth.stmms.biz.exam.service.ExamSubjectService;
import cn.com.qmth.stmms.biz.file.service.FileService;
import cn.com.qmth.stmms.biz.mark.model.PictureConfigItem;
import cn.com.qmth.stmms.biz.school.model.School;
import cn.com.qmth.stmms.biz.school.service.SchoolService;
import cn.com.qmth.stmms.biz.user.model.User;
import cn.com.qmth.stmms.biz.utils.PageUtil;
import cn.com.qmth.stmms.common.annotation.Logging;
import cn.com.qmth.stmms.common.domain.ApiUser;
import cn.com.qmth.stmms.common.enums.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import net.sf.json.JsonConfig;

@Api(tags = "考试管理")
@Controller("adminExamController")
@RequestMapping("/api/admin/exam")
public class ExamController extends BaseApiController {

    protected static final Logger log = LoggerFactory.getLogger(ExamController.class);

    @Autowired
    private ExamSubjectService subjectService;

    @Autowired
    private ExamStudentService studentService;

    @Autowired
    private SchoolService schoolService;

    @Autowired
    private ExamSubjectService examSubjectService;

    @Autowired
    private ExamStudentService examStudentService;

    @Autowired
    private ExamService examService;

    @Autowired
    private CheckStudentService checkStudentService;

    @Autowired
    private FileService fileService;

    @ApiOperation(value = "保存考试ai检测预估时间")
    @ResponseBody
    @RequestMapping(value = "answer-ai-check/finish-time", method = RequestMethod.POST)
    public ResultMessage answerAiCheckFinishTime(@RequestParam Integer examId, @RequestParam Long finishTime) {
        examService.answerAiCheckFinishTimeSave(examId, finishTime);
        return resultOk();
    }

    @ApiOperation(value = "获取考试ai检测预估时间")
    @ResponseBody
    @RequestMapping(value = "answer-ai-check/finish-time/info", method = RequestMethod.POST)
    public String answerAiCheckFinishTimeInfo(@RequestParam Integer examId) {
        Boolean finish = examStudentService.answerAiCheckFinish(examId);
        if (finish) {
            return "AI检测已完成";
        } else {
            Exam exam = examService.findById(examId);
            Long now = System.currentTimeMillis();
            if (exam.getAiCheckFinishTime() == null) {
                return "AI检测预计耗时预估中";
            } else {
                if (now >= exam.getAiCheckFinishTime().getTime()) {
                    return "AI检测预计耗时1分钟后完成";
                } else {
                    long time = exam.getAiCheckFinishTime().getTime() - now;
                    return "AI检测预计耗时" + getText(time) + "后完成";
                }
            }
        }
    }

    private String getText(long millis) {
        long totalSeconds = millis / 1000;
        long hours = totalSeconds / 3600;
        // 计算剩余秒数
        long remainingSecondsAfterHours = totalSeconds % 3600;
        // 计算分钟数
        long minutes = remainingSecondsAfterHours / 60;
        // 计算剩余秒数
        // int seconds = remainingSecondsAfterHours % 60;

        return String.format("%d小时%d分钟", hours, minutes);
    }
    // @Autowired
    // private SchoolService schoolService;

    // private Exam validateExam(ApiUser au, Integer examId, ExamType... types)
    // {
    // Exam exam = examService.findById(examId);
    // if (exam == null || !validateSchool(au.getSchoolId(), exam.getSchoolId())
    // || exam.getStatus() != ExamStatus.START) {
    // throw ApiException.EXAM_NOT_ACCESSIBLED;
    // }
    // if (types != null && types.length > 0 &&
    // !Arrays.asList(types).contains(exam.getType())) {
    // throw ApiException.QUERY_PARAM_ERROR.replaceMessage("exam type invalid");
    // }
    // return exam;
    // }

    @ApiOperation(value = "考试分页查询")
    @RequestMapping(value = "query", method = RequestMethod.POST)
    @ResponseBody
    public PageResult<ExamVo> query(ExamSearchQuery query) {
        ApiUser user = getApiUser();
        query.setSchoolId(user.getSchoolId());
        query.orderByIdDesc();
        query = examService.findByQuery(query);
        List<ExamVo> ret = new ArrayList<>();
        for (Exam e : query.getResult()) {
            ret.add(ExamVo.of(e));
        }
        return PageUtil.of(ret, query);
    }

    @ApiOperation(value = "考试类型")
    @RequestMapping(value = "types", method = RequestMethod.POST)
    @ResponseBody
    public List<ExamType> examTypeList() {
        return Arrays.asList(ExamType.values());
    }

    @ApiOperation(value = "考试状态")
    @RequestMapping(value = "status", method = RequestMethod.POST)
    @ResponseBody
    public List<ExamStatus> statusLlist() {
        return Arrays.asList(ExamStatus.values());
    }

    @ApiOperation(value = "考试信息查询")
    @ResponseBody
    @RequestMapping(value = "find", method = RequestMethod.POST)
    public ExamVo find(@RequestParam Integer examId) {
        Exam exam = examService.findById(examId);
        return ExamVo.of(exam);
    }

    @ApiOperation(value = "考试创建")
    @ResponseBody
    @Logging(menu = "创建考试", type = LogType.ADD)
    @RequestMapping(value = "save", method = RequestMethod.POST)
    public ResultMessage save(@RequestBody ExamVo vo) {
        User user = getApiUser().getUser();
        Exam exam = ExamVo.of(vo);
        exam.setSchoolId(user.getSchoolId());
        exam.setCreatorId(user.getId());
        exam.setStatus(ExamStatus.START);
        exam.setTrackCountPolicy(TrackCountPolicy.LE);
        exam.setObjectiveStatus(ObjectiveStatus.WAITING);
        if (exam.getType().equals(ExamType.MULTI_MEDIA)) {
            exam.setForceSpecialTag(false);
        }
        JSONObject sasConfig = new JSONObject();
        sasConfig.accumulate("passScore", vo.getPassScore());
        sasConfig.accumulate("excellentScore", vo.getExcellentScore());
        exam.setSasConfig(sasConfig.toString());
        exam = examService.save(exam);
        return resultOk();
    }

    @ApiOperation(value = "考试编辑")
    @ResponseBody
    @Logging(menu = "编辑考试", type = LogType.UPDATE)
    @SuppressWarnings("unchecked")
    @RequestMapping(value = "update", method = RequestMethod.POST)
    public ResultMessage edit(@RequestBody ExamVo exam) {
        User user = getApiUser().getUser();
        Exam oldExam = examService.findById(exam.getId());
        if (oldExam != null && oldExam.getSchoolId().equals(user.getSchoolId())) {
            oldExam.setName(exam.getName());
            oldExam.setExamTime(exam.getExamTime());
            oldExam.setDescription(exam.getDescription());
            oldExam.setStatus(exam.getStatus());
            oldExam.setForceSpecialTag(exam.getForceSpecialTag());
            oldExam.setForbiddenInfo(exam.getForbiddenInfo());
            if (exam.getForbiddenInfo()) {
                oldExam.setForbiddenScore(exam.getForbiddenScore());
            } else {
                oldExam.setForbiddenScore(false);
            }
            oldExam.setEnableSplit(exam.getEnableSplit());
            oldExam.setAutoScroll(exam.getAutoScroll());
            oldExam.setShowReject(exam.getShowReject());
            JSONObject sasConfig = new JSONObject();
            sasConfig.accumulate("passScore", exam.getPassScore());
            sasConfig.accumulate("excellentScore", exam.getExcellentScore());
            oldExam.setSasConfig(sasConfig.toString());
            if (StringUtils.isNotBlank(exam.getSheetConfig())) {
                String sheetConfig = exam.getSheetConfig();
                JSONArray array = JSONArray.fromObject(sheetConfig);
                List<PictureConfigItem> list = JSONArray.toList(array, new PictureConfigItem(), new JsonConfig());
                oldExam.setSheetConfig(list == null ? null : StringUtils.join(list, PictureConfigItem.DB_ITEM_JOINER));
            }
            oldExam.setStartTime(exam.getStartTime());
            oldExam.setEndTime(exam.getEndTime());
            oldExam.setMarkMode(exam.getMarkMode());
            oldExam.setSheetView(exam.getSheetView());
            oldExam.setInspectScrollBottom(exam.getInspectScrollBottom());
            oldExam.setInspectUnrepeated(exam.getInspectUnrepeated());
            oldExam.setShowObjectiveScore(exam.getShowObjectiveScore());
            oldExam.setRemarkCount(exam.getRemarkCount());
            oldExam.setInspectRoundLimit(exam.getInspectRoundLimit());
            oldExam.setTrackCountPolicy(exam.getTrackCountPolicy());
            oldExam.setMarkScrollBottom(exam.getMarkScrollBottom());
            oldExam.setMinMarkDuration(exam.getMinMarkDuration());
            oldExam.setReMarkLimitCount(exam.getReMarkLimitCount());
            oldExam.setBarcodeAiCheck(exam.getBarcodeAiCheck());
            oldExam.setAnswerAiCheck(exam.getAnswerAiCheck());
            examService.save(oldExam);
        }
        return resultOk();
    }

    @ApiOperation(value = "考试详情")
    @ResponseBody
    @Logging(menu = "考试详情", type = LogType.QUERY)
    @RequestMapping(value = "detail", method = RequestMethod.POST)
    public ExamView view(@RequestParam Integer examId) {
        ExamView ret = new ExamView();
        Exam exam = examService.findById(examId);
        long studentCount = examStudentService.countByExamId(exam.getId());
        long subjectCount = examSubjectService.count(examId);
        // long markerCount = markerService.countByExam(exam.getId());
        long scanCount = examStudentService.countByExamIdAndUpload(examId, true);
        long markedCount = examStudentService.countByExamIdAndSubjectiveScoreListIsNotNull(examId);
        long sheetCount = studentCount == 0 ? 0 : examStudentService.countSheetCountByExamId(examId);

        ret.setStudentCount(studentCount);

        ret.setSubjectCount(subjectCount);
        ret.setScanCount(scanCount);

        long sheetCount2 = new BigDecimal(sheetCount).divide(new BigDecimal(2), 0, BigDecimal.ROUND_HALF_UP)
                .longValue();

        ret.setPaperCount(sheetCount2);

        ret.setMarkedCount(markedCount);

        return ret;
    }

    @ApiOperation(value = "验收报告")
    @ResponseBody
    @RequestMapping(value = "report", method = RequestMethod.POST)
    public ExamReport report(@RequestParam Integer examId) {
        ExamReport ret = new ExamReport();
        Exam exam = examService.findById(examId);
        School school = schoolService.findById(exam.getSchoolId());
        ret.setSchoolName(school.getName());
        ret.setYear(Calendar.getInstance().get(Calendar.YEAR));
        ret.setSubjectCount(subjectService.count(examId));
        ret.setStudentCount(studentService.countByExamId(examId));
        ret.setSubjectiveStudentCount(studentService.countSubjectiveByExamId(examId));
        long totalPaperCount = studentService.countSheetCountByExamId(examId) / 2;
        ret.setObjectiveStudentCount(studentService.countObjectiveByExamId(examId));
        long objectivePaperCount = studentService.countObjectivePageCountByExamId(examId) / 2;
        ret.setSubjectivePaperCount(totalPaperCount - objectivePaperCount);
        ret.setObjectivePaperCount(objectivePaperCount);
        ret.setAllUploadStudentCount(studentService.countAllUploadByExamId(examId));
        ret.setTotalPaperCount(totalPaperCount);
        return ret;
    }

    @ApiOperation(value = "考试下拉列表查询")
    @RequestMapping(value = "list", method = RequestMethod.POST)
    @ResponseBody
    public List<ExamVo> list(ExamSearchQuery query) {
        ApiUser wu = getApiUser();
        List<Exam> examList = new ArrayList<Exam>();
        if (wu.isSubjectHeader() || wu.isInspector()) {
            examList = examService.findBySubjectHeaderUserId(wu.getUser().getSchoolId(), wu.getUser().getId());
        } else if (wu.isSchoolViewer()) {
            examList = examService.findBySchoolViewUserId(wu.getUser().getSchoolId(), wu.getUser().getId());
        } else if (wu.isMarker()) {
            Calendar rightNow = Calendar.getInstance();
            rightNow.setTime(new Date());
            rightNow.add(Calendar.YEAR, -1);
            examList = examService.findByMarkerUserId(wu.getUser().getId(), rightNow.getTime());
        } else {
            examList = examService.findBySchoolId(wu.getUser().getSchoolId());
        }
        List<ExamVo> ret = new ArrayList<>();
        for (Exam e : examList) {
            ret.add(ExamVo.of(e));
        }
        return ret;
    }

    @ApiOperation(value = "考试导览")
    @RequestMapping(value = "overview", method = RequestMethod.POST)
    @ResponseBody
    public ExamOverview overview() {
        int examId = getSessionExamId();
        ExamOverview overview = new ExamOverview();
        long studentCount = examStudentService.countByExamId(examId);
        overview.setStudentCount(studentCount);
        overview.setSubjectCount(examSubjectService.count(examId));
        if (studentCount > 0
                && checkStudentService.countByExamIdAndCheckedAndType(examId, false, CheckType.MANUAL) == 0) {
            overview.setCheckStudent(true);
        } else {
            overview.setCheckStudent(false);
        }
        overview.setCheckSubject(studentCount > 0 && examSubjectService.checkSubjectForExport(examId));
        overview.setCheckAbsent(studentCount > 0 && examSubjectService.checkSubjectAbsentForExport(examId));
        return overview;
    }

    @ApiOperation(value = "获取考试原图遮盖配置")
    @RequestMapping(value = "/sheetConfig", method = RequestMethod.POST)
    @ResponseBody
    public PictureConfigVo version(@RequestParam Integer examId) {
        PictureConfigVo vo = new PictureConfigVo();
        Exam exam = examService.findById(examId);
        if (exam == null) {
            throw new StatusException("考试不存在");
        }
        List<PictureConfigItem> list = PictureConfigItem.parse(exam.getSheetConfig());
        vo.setPictureConfigItemList(list);
        Integer sheetCount = studentService.findMaxSheetCountByExamId(examId);
        ExamStudentSearchQuery query = new ExamStudentSearchQuery();
        query.setExamId(examId);
        query.setUpload(true);
        query.setSheetCount(sheetCount/2);
        List<ExamStudent> examStudents = examStudentService.findByQuery(query).getResult();
        if (examStudents.size() > 0) {
            ExamStudent student = examStudents.get(0);
            List<String> sheetUris= fileService.getSheetUris(student.getExamId(), student.getExamNumber(), 1, sheetCount);
            vo.setStudentUrl(sheetUris);
            return vo;
        }else{
            throw new StatusException("没有已上传的考生");
        }
    }


}
