package cn.com.qmth.stmms.api.controller.admin;

import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import cn.com.qmth.stmms.api.controller.BaseApiController;
import com.qmth.boot.core.exception.StatusException;
import cn.com.qmth.stmms.biz.mark.model.ProblemType;
import cn.com.qmth.stmms.biz.mark.service.ProblemHistoryService;
import cn.com.qmth.stmms.biz.mark.service.ProblemTypeService;
import cn.com.qmth.stmms.common.annotation.Logging;
import cn.com.qmth.stmms.common.domain.ApiUser;
import cn.com.qmth.stmms.common.enums.LogType;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@Api(tags = "问题卷分类管理")
@Controller("adminProblemTypeController")
@RequestMapping("/api/admin/exam/problem/type")
public class ProblemTypeController extends BaseApiController {

    protected static Logger log = LoggerFactory.getLogger(ProblemTypeController.class);

    @Autowired
    private ProblemTypeService problemService;

    @Autowired
    private ProblemHistoryService historyService;

    @ApiOperation(value = "查询")
    @Logging(menu = "查询问题卷分类", type = LogType.QUERY)
    @RequestMapping(value = "list", method = RequestMethod.POST)
    @ResponseBody
    public List<ProblemType> list(@RequestParam Integer examId) {
        return problemService.findByExamId(examId);
    }

    @ApiOperation(value = "新增")
    @Logging(menu = "新增问题卷分类", type = LogType.ADD)
    @RequestMapping(value = "save", method = RequestMethod.POST)
    @ResponseBody
    public void save(ProblemType problem) {
        if (problem.getExamId() == null) {
            throw new StatusException("examId不能为空");
        }
        if (!StringUtils.isNotBlank(problem.getName())) {
            throw new StatusException("名称不能为空");
        }
        Integer examId = problem.getExamId();
        List<ProblemType> list = problemService.findByExamId(examId);
        if (list.size() >= 10) {
            throw new StatusException("问题卷类型不能超过10个");
        }
        problem.setCustom(true);
        problem = problemService.save(problem);
    }

    @ApiOperation(value = "查询信息")
    @RequestMapping(value = "find", method = RequestMethod.POST)
    @ResponseBody
    public ProblemType find(@PathVariable Integer id) {
        return problemService.findById(id);
    }

    @ApiOperation(value = "修改")
    @Logging(menu = "编辑问题卷分类", type = LogType.UPDATE)
    @RequestMapping(value = "update", method = RequestMethod.POST)
    @ResponseBody
    public void update(ProblemType problem) {
        ApiUser user = getApiUser();
        ProblemType old = problemService.findById(problem.getId());
        if (user.isSchoolAdmin() && StringUtils.isNotBlank(problem.getName())) {
            old.setName(problem.getName());
            problemService.save(old);
        }
    }

    @ApiOperation(value = "删除")
    @Logging(menu = "删除问题卷分类", type = LogType.DELETE)
    @RequestMapping(value = "delete", method = RequestMethod.POST)
    @ResponseBody
    public void delete(@RequestParam Integer id) {
        int examId = getSessionExamId();
        long historyCount = historyService.countByExamIdAndProblemId(examId, id);
        if (historyCount > 0) {
            throw new StatusException("该问题卷类型已被使用，不可删除");
        } else {
            problemService.deleteById(id);
        }
    }
}
